@@ -1,5 +1,5 @@
 #
-# $Id: Tool.pm,v b5a7d96de9f1 2016/10/11 06:17:23 gomor $
+# $Id: Tool.pm,v 94e85541d72a 2017/02/04 16:34:12 gomor $
 #
 # brik::tool Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik::Shell::Command);
 
 sub brik_properties {
    return {
-      revision => '$Revision: b5a7d96de9f1 $',
+      revision => '$Revision: 94e85541d72a $',
       tags => [ qw(unstable program) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -45,7 +45,17 @@ sub brik_properties {
          view_brik_source => [ qw(Brik) ],
          get_brik_module_file => [ qw(Brik directory_list|OPTIONAL) ],
          clone => [ qw(Brik Repository|OPTIONAL) ],
+         get_require_binaries => [ qw(Brik|OPTIONAL) ],
       },
+      # We can't activate that, because we would have a chicken-and-egg problem.
+      #need_packages => {
+         #ubuntu => [ qw(mercurial) ],
+         #debian => [ qw(mercurial) ],
+         #freebsd => [ qw(mercurial) ],
+      #},
+      #require_binaries => {
+         #hg => [ ],
+      #},
       require_modules => {
          'Metabrik::Devel::Mercurial' => [ ],
          'Metabrik::File::Find' => [ ],
@@ -332,7 +342,7 @@ sub install_all_need_packages {
    }
 
    if (@missing > 0) {
-      return $self->log->error("install_all_need_packages: unable to install packages for ".
+      $self->log->warning("install_all_need_packages: unable to install packages for ".
          "Brik(s): [".join(', ', @missing)."]");
    }
 
@@ -387,6 +397,17 @@ sub install {
    $self->install_needed_packages($brik) or return;
    $self->install_required_modules($brik) or return;
 
+   my $module = 'Metabrik';
+   my @toks = split(/::/, $brik);
+   for (@toks) {
+      $module .= '::'.ucfirst($_);
+   }
+
+   my $new = $module->new_from_brik_no_checks($self) or return;
+   if ($new->can('install')) {
+      $new->install or return;
+   }
+
    return 1;
 }
 
@@ -567,7 +588,7 @@ $package - $brik Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
@@ -681,10 +702,15 @@ sub view_brik_source {
    $self->brik_help_run_undef_arg('view_brik_source', $brik) or return;
 
    my @toks = split(/::/, $brik);
-   if (@toks < 2) {
+   if (@toks < 2 && $brik ne 'metabrik') {
       return $self->log->error("view_brik_source: invalid Brik format for [$brik]");
    }
 
+   # Handle special case for Metabrik.pm
+   if ($brik eq 'metabrik') {
+      @toks = ();
+   }
+
    my $pager = $ENV{PAGER} || 'less';
 
    my $pm = 'Metabrik';
@@ -694,8 +720,11 @@ sub view_brik_source {
    }
    $pm .= '.pm';
 
+   $self->log->debug("view_brik_source: pm [$pm]");
+
    my $cmd = '';
    for (@INC) {
+      $self->log->debug("view_brik_source: search [$_/$pm] file");
       if (-f "$_/$pm") {
          $cmd = "$pager $_/$pm";
          last;
@@ -805,6 +834,40 @@ sub clone {
    return $dst_file;
 }
 
+#
+# Will return the complete list of required binaries if no Argument is given,
+# or the list of required binaries for the specified Brik.
+#
+sub get_require_binaries {
+   my $self = shift;
+   my ($brik) = @_;
+
+   my $con = $self->context;
+   my $available = $con->available;
+
+   # If we asked for one Brik, we rewrite available to only have this one.
+   if (defined($brik)) {
+      $available = { $brik => $available->{$brik} };
+   }
+
+   my $sp = Metabrik::System::Package->new_from_brik_init($self) or return;
+   my $os = $sp->my_os or return;
+
+   my %packages = ();
+   for my $this (keys %$available) {
+      next if $this =~ m{^core::};
+      if (defined($available->{$this})
+      &&  exists($available->{$this}->brik_properties->{require_binaries})) {
+         my $list = [ keys %{$available->{$this}->brik_properties->{require_binaries}} ];
+         for my $p (@$list) {
+            $packages{$p}++;
+         }
+      }
+   }
+
+   return [ sort { $a cmp $b } keys %packages ];
+}
+
 1;
 
 __END__
@@ -815,7 +878,7 @@ Metabrik::Brik::Tool - brik::tool Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
