@@ -1,5 +1,5 @@
 #
-# $Id: Query.pm,v 7ece15bd5955 2016/04/16 15:16:53 gomor $
+# $Id: Query.pm,v 84f8ea395fe8 2017/01/10 14:59:25 gomor $
 #
 # client::elasticsearch::query Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik::Client::Elasticsearch);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 7ece15bd5955 $',
+      revision => '$Revision: 84f8ea395fe8 $',
       tags => [ qw(unstable) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -30,15 +30,20 @@ sub brik_properties {
       commands => {
          create_client => [ ],
          reset_client => [ ],
-         get_query_result_total => [ qw($query_result) ],
-         get_query_result_hits => [ qw($query_result) ],
-         get_query_result_timed_out => [ qw($query_result) ],
-         get_query_result_took => [ qw($query_result) ],
+         query => [ qw(query index|OPTIONAL type|OPTIONAL) ],
+         get_query_result_total => [ qw($query_result|OPTIONAL) ],
+         get_query_result_hits => [ qw($query_result|OPTIONAL) ],
+         get_query_result_timed_out => [ qw($query_result|OPTIONAL) ],
+         get_query_result_took => [ qw($query_result|OPTIONAL) ],
          term => [ qw(kv index|OPTIONAL type|OPTIONAL) ],
+         unique_term => [ qw(unique kv index|OPTIONAL type|OPTIONAL) ],
          wildcard => [ qw(kv index|OPTIONAL type|OPTIONAL) ],
          range => [ qw(kv_from kv_to index|OPTIONAL type|OPTIONAL) ],
          top => [ qw(kv_count index|OPTIONAL type|OPTIONAL) ],
          top_match => [ qw(kv_count kv_match index|OPTIONAL type|OPTIONAL) ],
+         match => [ qw(kv index|OPTIONAL type|OPTIONAL) ],
+         match_phrase => [ qw(kv index|OPTIONAL type|OPTIONAL) ],
+         from_json_file => [ qw(json_file index|OPTIONAL type|OPTIONAL) ],
       },
    };
 }
@@ -58,13 +63,20 @@ sub create_client {
 sub reset_client {
    my $self = shift;
 
-   return $self->log->info("TODO");
+   my $ce = $self->client;
+   if (defined($ce)) {
+      $self->client(undef);
+   }
+
+   return 1;
 }
 
 sub get_query_result_total {
    my $self = shift;
    my ($query_result) = @_;
 
+   my $run = $self->context->do('$RUN');
+   $query_result ||= $run;
    $self->brik_help_run_undef_arg('get_query_result_total', $query_result) or return;
    $self->brik_help_run_invalid_arg('get_query_result_total', $query_result, 'HASH') or return;
 
@@ -82,6 +94,8 @@ sub get_query_result_hits {
    my $self = shift;
    my ($query_result) = @_;
 
+   my $run = $self->context->do('$RUN');
+   $query_result ||= $run;
    $self->brik_help_run_undef_arg('get_query_result_hits', $query_result) or return;
    $self->brik_help_run_invalid_arg('get_query_result_hits', $query_result, 'HASH') or return;
 
@@ -99,6 +113,8 @@ sub get_query_result_timed_out {
    my $self = shift;
    my ($query_result) = @_;
 
+   my $run = $self->context->do('$RUN');
+   $query_result ||= $run;
    $self->brik_help_run_undef_arg('get_query_result_timed_out', $query_result) or return;
    $self->brik_help_run_invalid_arg('get_query_result_timed_out', $query_result, 'HASH')
       or return;
@@ -115,6 +131,8 @@ sub get_query_result_took {
    my $self = shift;
    my ($query_result) = @_;
 
+   my $run = $self->context->do('$RUN');
+   $query_result ||= $run;
    $self->brik_help_run_undef_arg('get_query_result_took', $query_result) or return;
    $self->brik_help_run_invalid_arg('get_query_result_took', $query_result, 'HASH')
       or return;
@@ -126,21 +144,27 @@ sub get_query_result_took {
    return $query_result->{took};
 }
 
-sub _query {
+sub query {
    my $self = shift;
    my ($q, $index, $type) = @_;
 
-   my $r = $self->query($q, $index, $type) or return;
+   $index ||= '*';
+   $type ||= '*';
+   $self->brik_help_run_undef_arg('query', $q) or return;
+
+   my $ce = $self->create_client or return;
+
+   my $r = $self->SUPER::query($q, $index, $type) or return;
    if (defined($r)) {
       if (exists($r->{hits}{total})) {
          return $r;
       }
       else {
-         return $self->log->error("_query: failed with [$r]");
+         return $self->log->error("query: failed with [$r]");
       }
    }
 
-   return $self->log->error("_query: failed");
+   return $self->log->error("query: failed");
 }
 
 #
@@ -153,33 +177,88 @@ sub term {
    $index ||= $self->index;
    $type ||= $self->type;
    $self->brik_help_run_undef_arg('term', $kv) or return;
-   $self->brik_help_run_undef_arg('term', $index) or return;
-   $self->brik_help_run_undef_arg('term', $type) or return;
+   $self->brik_help_set_undef_arg('term', $index) or return;
+   $self->brik_help_set_undef_arg('term', $type) or return;
 
-   if ($kv !~ /^\S+=\S+$/) {
+   if ($kv !~ /^\S+?=.+$/) {
       return $self->log->error("term: kv must be in the form 'key=value'");
    }
    my ($key, $value) = split('=', $kv);
 
    $self->debug && $self->log->debug("term: key[$key] value[$value]");
 
-   my $ce = $self->create_client or return;
-
+   # Optimized version on ES 5.0.0
    my $q = {
       query => {
-         constant_score => { 
-            filter => {
-               term => {
-                  $key => $value,
-               },
+         bool => {
+            must => { term => { $key => $value } },
+         },
+      },
+   };
+
+   $self->log->verbose("term: keys [$key] value [$value] index [$index] type [$type]");
+
+   return $self->query($q, $index, $type);
+}
+
+#
+# run client::elasticsearch::query unique_term ip domain=example.com index1-*,index2-*
+#
+sub unique_term {
+   my $self = shift;
+   my ($unique, $kv, $index, $type) = @_;
+
+   $index ||= $self->index;
+   $type ||= $self->type;
+   $self->brik_help_run_undef_arg('term', $unique) or return;
+   $self->brik_help_run_undef_arg('term', $kv) or return;
+   $self->brik_help_set_undef_arg('term', $index) or return;
+   $self->brik_help_set_undef_arg('term', $type) or return;
+
+   if ($kv !~ /^\S+?=.+$/) {
+      return $self->log->error("unique_term: kv must be in the form 'key=value'");
+   }
+   my ($key, $value) = split('=', $kv);
+
+   $self->debug && $self->log->debug("unique_term: key[$key] value[$value]");
+
+   # Optimized version on ES 5.0.0
+   my $q = {
+      size => 0,
+      query => {
+         bool => {
+            must => { term => { $key => $value } },
+         },
+      },
+      aggs => {
+         1 => {
+            cardinality => {
+               field => $unique,
             },
          },
       },
    };
 
-   return $self->_query($q, $index, $type);
+   $self->log->verbose("unique_term: unique [$unique] keys [$key] value [$value] ".
+      "index [$index] type [$type]");
+
+   return $self->query($q, $index, $type);
 }
 
+#
+# 
+#
+#sub unique_values {
+#{
+   #"size": 0,
+   #"aggs": {
+      #"1" : {
+         #"terms" : { "field" : "ip" },
+      #}
+   #}
+#}
+#} 
+
 sub wildcard {
    my $self = shift;
    my ($kv, $index, $type) = @_;
@@ -187,16 +266,14 @@ sub wildcard {
    $index ||= $self->index;
    $type ||= $self->type;
    $self->brik_help_run_undef_arg('wildcard', $kv) or return;
-   $self->brik_help_run_undef_arg('wildcard', $index) or return;
-   $self->brik_help_run_undef_arg('wildcard', $type) or return;
+   $self->brik_help_set_undef_arg('wildcard', $index) or return;
+   $self->brik_help_set_undef_arg('wildcard', $type) or return;
 
-   if ($kv !~ /^\S+=\S+$/) {
+   if ($kv !~ /^\S+?=.+$/) {
       return $self->log->error("wildcard: kv must be in the form 'key=value'");
    }
    my ($key, $value) = split('=', $kv);
 
-   my $ce = $self->create_client or return;
-
    my $q = {
       query => {
          #constant_score => {  # Does not like constant_score
@@ -209,8 +286,9 @@ sub wildcard {
       },
    };
 
-   return $self->_query($q, $index, $type);
+   $self->log->verbose("wildcard: keys [$key] value [$value] index [$index] type [$type]");
 
+   return $self->query($q, $index, $type);
 }
 
 #
@@ -224,34 +302,35 @@ sub range {
    $type ||= $self->type;
    $self->brik_help_run_undef_arg('range', $kv_from) or return;
    $self->brik_help_run_undef_arg('range', $kv_to) or return;
-   $self->brik_help_run_undef_arg('range', $index) or return;
-   $self->brik_help_run_undef_arg('range', $type) or return;
+   $self->brik_help_set_undef_arg('range', $index) or return;
+   $self->brik_help_set_undef_arg('range', $type) or return;
 
-   if ($kv_from !~ /^\S+=\S+$/) {
+   if ($kv_from !~ /^\S+?=.+$/) {
       return $self->log->error("range: kv_from [$kv_from] must be in the form 'key=value'");
    }
-   if ($kv_to !~ /^\S+=\S+$/) {
+   if ($kv_to !~ /^\S+?=.+$/) {
       return $self->log->error("range: kv_to [$kv_to] must be in the form 'key=value'");
    }
    my ($key_from, $value_from) = split('=', $kv_from);
    my ($key_to, $value_to) = split('=', $kv_to);
 
-   my $ce = $self->create_client or return;
-
+   #
+   # http://stackoverflow.com/questions/40519806/no-query-registered-for-filtered
+   # https://www.elastic.co/guide/en/elasticsearch/reference/current/query-dsl-filtered-query.html
+   # Compatible with ES 5.0
+   #
    my $q = {
       query => {
-         constant_score => {
-            filter => {
-               and => [
-                  { range => { $key_to => { gte => $value_to } } },
-                  { range => { $key_from => { lte => $value_from } } },
-               ],
-            },
+         bool => {
+            must => [
+               { range => { $key_to => { gte => $value_to } } },
+               { range => { $key_from => { lte => $value_from } } },
+            ],
          },
       },
    };
 
-   return $self->_query($q, $index, $type);
+   return $self->query($q, $index, $type);
 }
 
 #
@@ -264,16 +343,14 @@ sub top {
    $index ||= $self->index;
    $type ||= $self->type;
    $self->brik_help_run_undef_arg('top', $kv_count) or return;
-   $self->brik_help_run_undef_arg('top', $index) or return;
-   $self->brik_help_run_undef_arg('top', $type) or return;
+   $self->brik_help_set_undef_arg('top', $index) or return;
+   $self->brik_help_set_undef_arg('top', $type) or return;
 
-   if ($kv_count !~ /^\S+=\S+$/) {
+   if ($kv_count !~ /^\S+=\d+$/) {
       return $self->log->error("top: kv_count [$kv_count] must be in the form 'key=value'");
    }
    my ($key_count, $value_count) = split('=', $kv_count);
 
-   my $ce = $self->create_client or return;
-
    my $q = {
       aggs => {
          top_values => {
@@ -286,7 +363,65 @@ sub top {
       },
    };
 
-   return $self->_query($q, $index, $type);
+   $self->log->verbose("top: key [$key_count] value [$value_count] index [$index] type [$type]");
+
+   return $self->query($q, $index, $type);
+}
+
+sub match_phrase {
+   my $self = shift;
+   my ($kv, $index, $type) = @_;
+
+   $index ||= $self->index;
+   $type ||= $self->type;
+   $self->brik_help_run_undef_arg('match_phrase', $kv) or return;
+   $self->brik_help_set_undef_arg('match_phrase', $index) or return;
+   $self->brik_help_set_undef_arg('match_phrase', $type) or return;
+
+   if ($kv !~ /^\S+?=.+$/) {
+      return $self->log->error("match_phrase: kv must be in the form 'key=value'");
+   }
+   my ($key, $value) = split('=', $kv);
+
+   $self->debug && $self->log->debug("match_phrase: key[$key] value[$value]");
+
+   my $q = {
+      query => {
+         match_phrase => {
+            $key => $value,
+         },
+      },
+   };
+
+   return $self->query($q, $index, $type);
+}
+
+sub match {
+   my $self = shift;
+   my ($kv, $index, $type) = @_;
+
+   $index ||= $self->index;
+   $type ||= $self->type;
+   $self->brik_help_run_undef_arg('match', $kv) or return;
+   $self->brik_help_set_undef_arg('match', $index) or return;
+   $self->brik_help_set_undef_arg('match', $type) or return;
+
+   if ($kv !~ /^\S+?=.+$/) {
+      return $self->log->error("match: kv must be in the form 'key=value'");
+   }
+   my ($key, $value) = split('=', $kv);
+
+   $self->debug && $self->log->debug("match: key[$key] value[$value]");
+
+   my $q = {
+      query => {
+         match => {
+            $key => $value,
+         },
+      },
+   };
+
+   return $self->query($q, $index, $type);
 }
 
 #
@@ -300,20 +435,18 @@ sub top_match {
    $type ||= $self->type;
    $self->brik_help_run_undef_arg('top_match', $kv_count) or return;
    $self->brik_help_run_undef_arg('top_match', $kv_match) or return;
-   $self->brik_help_run_undef_arg('top_match', $index) or return;
-   $self->brik_help_run_undef_arg('top_match', $type) or return;
+   $self->brik_help_set_undef_arg('top_match', $index) or return;
+   $self->brik_help_set_undef_arg('top_match', $type) or return;
 
-   if ($kv_count !~ /^\S+=\S+$/) {
+   if ($kv_count !~ /^\S+?=.+$/) {
       return $self->log->error("top_match: kv_count [$kv_count] must be in the form 'key=value'");
    }
-   if ($kv_match !~ /^\S+=\S+$/) {
+   if ($kv_match !~ /^\S+?=.+$/) {
       return $self->log->error("top_match: kv_match [$kv_match] must be in the form 'key=value'");
    }
    my ($key_count, $value_count) = split('=', $kv_count);
    my ($key_match, $value_match) = split('=', $kv_match);
 
-   my $ce = $self->create_client or return;
-
    my $q = {
       query => {
          #constant_score => {   # Does not like constant_score
@@ -334,7 +467,24 @@ sub top_match {
       },
    };
 
-   return $self->_query($q, $index, $type);
+   return $self->query($q, $index, $type);
+}
+
+sub from_json_file {
+   my $self = shift;
+   my ($file, $index, $type) = @_;
+
+   $index ||= $self->index;
+   $type ||= $self->type;
+   $self->brik_help_run_undef_arg('from_json_file', $file) or return;
+   $self->brik_help_run_file_not_found('from_json_file', $file) or return;
+   $self->brik_help_set_undef_arg('from_json_file', $index) or return;
+   $self->brik_help_set_undef_arg('from_json_file', $type) or return;
+
+   my $fj = Metabrik::File::Json->new_from_brik_init($self) or return;
+   my $q = $fj->read($file) or return;
+
+   return $self->query($q, $index, $type);
 }
 
 1;
@@ -347,7 +497,7 @@ Metabrik::Client::Elasticsearch::Query - client::elasticsearch::query Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
