@@ -1,5 +1,5 @@
 #
-# $Id: Rest.pm,v 75fc1a5bf385 2016/09/28 05:57:21 gomor $
+# $Id: Rest.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # client::rest Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik::Client::Www);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 75fc1a5bf385 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable http api) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -65,7 +65,8 @@ sub content {
       return $self->log->error("content: output_mode not supported [$output_mode]");
    }
 
-   return $sm->decode($last->content);
+   # We must decode content cause it may be gzipped, for instance.
+   return $sm->decode($last->decoded_content);
 }
 
 1;
@@ -78,7 +79,7 @@ Metabrik::Client::Rest - client::rest Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
