@@ -1,5 +1,5 @@
 #
-# $Id: Git.pm,v fc658e60fb76 2016/10/08 15:24:49 gomor $
+# $Id: Git.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # devel::git Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik::Shell::Command Metabrik::System::Package);
 
 sub brik_properties {
    return {
-      revision => '$Revision: fc658e60fb76 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -29,6 +29,7 @@ sub brik_properties {
       commands => {
          install => [ ], # Inherited
          clone => [ qw(repository directory|OPTIONAL) ],
+         update => [ qw(repository directory|OPTIONAL) ],
       },
       require_binaries => {
          git => [ ],
@@ -61,6 +62,27 @@ sub clone {
    return $directory;
 }
 
+sub update {
+   my $self = shift;
+   my ($repository, $directory) = @_;
+
+   $self->brik_help_run_undef_arg('update', $repository) or return;
+
+   if (! defined($directory)) {
+      my $datadir = $self->datadir;
+
+      $directory ||= $datadir;
+      my ($name) = $repository =~ m{^.*/(.*)$};
+      $directory .= '/'.$name;
+   }
+
+   my $cmd = "git pull -u $repository $directory";
+
+   $self->execute($cmd) or return;
+
+   return $directory;
+}
+
 1;
 
 __END__
@@ -71,7 +93,7 @@ Metabrik::Devel::Git - devel::git Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
