@@ -1,5 +1,5 @@
 #
-# $Id: Csv.pm,v 1ae91f344f91 2016/04/17 14:10:32 gomor $
+# $Id: Csv.pm,v 1e1671f227f5 2017/01/05 17:29:15 gomor $
 #
 # file::csv Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 1ae91f344f91 $',
+      revision => '$Revision: 1e1671f227f5 $',
       tags => [ qw(unstable) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -20,23 +20,29 @@ sub brik_properties {
          output => [ qw(file) ],
          first_line_is_header => [ qw(0|1) ],
          separator => [ qw(character) ],
+         escape => [ qw(character) ],
          header => [ qw($column_header_list) ],
          encoding => [ qw(utf8|ascii) ],
          overwrite => [ qw(0|1) ],
          append => [ qw(0|1) ],
          write_header => [ qw(0|1) ],
          use_quoting => [ qw(0|1) ],
+         use_locking => [ qw(0|1) ],
+         unbuffered => [ qw(0|1) ],
          _csv => [ qw(INTERNAL) ],
          _fd => [ qw(INTERNAL) ],
       },
       attributes_default => {
          first_line_is_header => 1,
          separator => ',',
+         escape => '"',
          encoding => 'utf8',
-         overwrite => 1,
-         append => 0,
+         overwrite => 0,
+         append => 1,
          write_header => 1,
          use_quoting => 0,
+         use_locking => 0,
+         unbuffered => 0,
       },
       commands => {
          read => [ qw(input_file|OPTIONAL) ],
@@ -65,6 +71,7 @@ sub read {
       sep_char => $self->separator,
       allow_loose_quotes => 1,
       allow_loose_escapes => 1,
+      escape_char => $self->escape,
    }) or return $self->log->error('read: Text::CSV_XS new failed');
 
    my $fr = Metabrik::File::Read->new_from_brik_init($self) or return;
@@ -132,6 +139,8 @@ sub write {
    $fw->encoding($self->encoding);
    $fw->overwrite($self->overwrite);
    $fw->append($self->append);
+   $fw->use_locking($self->use_locking);
+   $fw->unbuffered($self->unbuffered);
 
    #
    # Set header ordering
@@ -164,9 +173,12 @@ sub write {
    my $written = '';
 
    # Write header if this is a new file and user asked for it.
-   if ($self->write_header && $is_new_file) {
+   if ($self->write_header && ($is_new_file || $self->overwrite)) {
       my $data = join($self->separator, @header)."\n";
-      print $fd $data;
+      my $r = $fw->write($data);
+      if (! defined($r)) {
+         return;
+      }
       $written .= $data;
    }
 
@@ -262,6 +274,7 @@ sub read_next {
          sep_char => $self->separator,
          allow_loose_quotes => 1,
          allow_loose_escapes => 1,
+         escape_char => $self->escape,
       }) or return $self->log->error('read_next: Text::CSV_XS new failed');
       $self->_csv($csv);
 
@@ -309,7 +322,7 @@ Metabrik::File::Csv - file::csv Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
