@@ -1,5 +1,5 @@
 #
-# $Id: Dual.pm,v b854c1d9459a 2016/04/11 17:14:14 gomor $
+# $Id: Dual.pm,v 014bf41a7d83 2017/01/21 11:08:30 gomor $
 #
 # log::dual Brik
 #
@@ -11,11 +11,12 @@ use base qw(Metabrik::Core::Log);
 
 sub brik_properties {
    return {
-      revision => '$Revision: b854c1d9459a $',
+      revision => '$Revision: 014bf41a7d83 $',
       tags => [ qw(unstable logging) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
       attributes => {
+         datadir => [ qw(datadir) ],
          level => [ qw(0|1|2|3) ],
          output => [ qw(file) ],
          time_prefix => [ qw(0|1) ],
@@ -59,7 +60,7 @@ sub brik_init {
    my $self = shift;
 
    my $output = $self->output;
-   open(my $fd, '>', $output)
+   open(my $fd, '>>', $output)
       or return $self->log->error("brik_init: can't open output file [$output]: $!");
 
    # Makes the file handle unbuffered
@@ -203,7 +204,7 @@ Metabrik::Log::Dual - log::dual Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
