@@ -1,5 +1,5 @@
 #
-# $Id: Iplocation.pm,v 811ac7ccde81 2016/02/22 18:15:08 gomor $
+# $Id: Iplocation.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # lookup::iplocation Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 811ac7ccde81 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable location ipv4 ipv6 ip geo geolocation) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -25,6 +25,8 @@ sub brik_properties {
          from_ipv6 => [ qw(ipv6_address) ],
          subnet4 => [ qw(ipv4_address) ],
          organization_name => [ qw(ip_address) ],
+         range_from_ipv4 => [ qw(ipv4_address) ],
+         networks_from_ipv4 => [ qw(ipv4_address) ],
       },
       require_modules => {
          'Geo::IP' => [ ],
@@ -76,13 +78,66 @@ sub from_ipv4 {
 
    $self->brik_help_run_undef_arg('from_ipv4', $ipv4) or return;
 
+   my $na = Metabrik::Network::Address->new_from_brik_init($self) or return;
+
    my $gi = Geo::IP->open($self->datadir.'/GeoIPCity.dat', Geo::IP::GEOIP_STANDARD())
       or return $self->log->error("from_ipv4: unable to open GeoIPCity.dat");
 
-   my $record = $gi->record_by_addr($ipv4);
+   my $gi_asn = Geo::IP->open($self->datadir.'/GeoIPASNum.dat', Geo::IP::GEOIP_STANDARD())
+      or return $self->log->error("from_ipv4: unable to open GeoIPASNum.dat");
+
+   my $record;
+   eval {
+      $record = $gi->record_by_addr($ipv4);
+   };
+   if ($@ || ! defined($record)) {
+      chomp($@);
+      return $self->log->error("from_ipv4: unable to find info for IPv4 [$ipv4] ".
+         "with error [$@]");
+   }
 
    # Convert from blessed hashref to hashref
-   return { map { $_ => $record->{$_} } keys %$record };
+   my $h = { map { $_ => $record->{$_} } keys %$record };
+   $h->{timezone} = $record->time_zone;
+
+   my $asn = '';
+   my $organization = '';
+   my $asn_organization = $gi_asn->name_by_addr($ipv4);
+   if ($asn_organization) {
+      ($asn, $organization) = $asn_organization =~ m{^(\S+)(?:\s+(.*))?$};
+      $asn ||= $asn_organization;  # Not able to parse, we put it raw.
+   }
+   $asn ||= 'undef';
+   $organization ||= 'undef';
+
+   my ($from, $to) = $gi->range_by_ip($ipv4);
+   if (! defined($from) || ! defined($to)) {
+      return $self->log->error("from_ipv4: unable to find range for IPv4 [$ipv4]");
+   }
+
+   my $network = $na->range_to_cidr($from, $to) or return;
+   my $network_list = join('|', @$network);
+
+   # Add other info and return
+   $h->{asn} = $asn;
+   $h->{organization} = $organization;
+   $h->{first_ip} = $from;
+   $h->{last_ip} = $to;
+   $h->{networks} = $network_list;
+
+   # If not defined, we set to 0, as this should be a number.
+   $h->{dma_code} ||= 0;
+   $h->{area_code} ||= 0;
+   $h->{metro_code} ||= 0;
+
+   # Set as undef if nothing found
+   for my $k (keys %$h) {
+      if (! defined($h->{$k}) || ! length($h->{$k})) {
+         $h->{$k} = 'undef';
+      }
+   }
+
+   return $h;
 }
 
 sub from_ipv6 {
@@ -91,6 +146,10 @@ sub from_ipv6 {
 
    $self->brik_help_run_undef_arg('from_ipv6', $ipv6) or return;
 
+   # XXX: IPv6:
+   # my $gi = Geo::IP->open( "/usr/local/share/GeoIP/GeoIPASNumv6.dat", GEOIP_STANDARD );
+   # print $gi->name_by_addr_v6('::ffff:24.24.24.24') || '';
+
    my $gi = Geo::IP->open($self->datadir.'/GeoIPv6.dat')
       or return $self->log->error("from_ipv6: unable to open GeoIPv6.dat");
 
@@ -147,6 +206,33 @@ sub organization_name {
    return $record;
 }
 
+sub range_from_ipv4 {
+   my $self = shift;
+   my ($ipv4) = @_;
+
+   $self->brik_help_run_undef_arg('range_from_ipv4', $ipv4) or return;
+
+   my $gi = Geo::IP->open($self->datadir.'/GeoIPCity.dat', Geo::IP::GEOIP_STANDARD())
+      or return $self->log->error("range_from_ipv4: unable to open GeoIPCity.dat");
+
+   my ($from, $to) = $gi->range_by_ip($ipv4);
+
+   return [ $from, $to ];
+}
+
+sub networks_from_ipv4 {
+   my $self = shift;
+   my ($ipv4) = @_;
+
+   $self->brik_help_run_undef_arg('networks_from_ipv4', $ipv4) or return;
+
+   my $range = $self->range_from_ipv4($ipv4) or return;
+
+   my $na = Metabrik::Network::Address->new_from_brik_init($self) or return;
+
+   return $na->range_to_cidr($range->[0], $range->[1]);
+}
+
 1;
 
 __END__
@@ -157,7 +243,7 @@ Metabrik::Lookup::Iplocation - lookup::iplocation Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
