@@ -1,5 +1,5 @@
 #
-# $Id: Address.pm,v c439ed2e7993 2016/04/04 18:06:06 gomor $
+# $Id: Address.pm,v b8ac01b4c14b 2017/01/21 14:38:39 gomor $
 #
 # network::address Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: c439ed2e7993 $',
+      revision => '$Revision: b8ac01b4c14b $',
       tags => [ qw(unstable netmask convert ascii) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -368,6 +368,12 @@ sub ipv4_to_integer {
 
    $self->brik_help_run_undef_arg('ipv4_to_integer', $ipv4_address) or return;
 
+   if (! $self->is_ipv4($ipv4_address)) {
+      return $self->log->error("ipv4_to_integer: invalid IPv4 address [$ipv4_address]");
+   }
+
+   ($ipv4_address) =~ s/\/\d+$//;  # Remove /CIDR if any
+
    return CORE::unpack('N', Socket::inet_aton($ipv4_address));
 }
 
@@ -390,7 +396,7 @@ Metabrik::Network::Address - network::address Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
