@@ -1,5 +1,5 @@
 #
-# $Id: Winexe.pm,v fc658e60fb76 2016/10/08 15:24:49 gomor $
+# $Id: Winexe.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # remote::winexe Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik::Shell::Command Metabrik::System::Package);
 
 sub brik_properties {
    return {
-      revision => '$Revision: fc658e60fb76 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -26,6 +26,7 @@ sub brik_properties {
       commands => {
          install => [ ], # Inherited
          execute => [ qw(command host|OPTIONAL user|OPTIONAL password|OPTIONAL) ],
+         execute_in_background => [ qw(command host|OPTIONAL user|OPTIONAL password|OPTIONAL) ],
       },
       require_modules => {
          'Metabrik::System::File' => [ ],
@@ -126,7 +127,26 @@ sub execute {
    $self->brik_help_run_undef_arg('execute', $user) or return;
    $self->brik_help_run_undef_arg('execute', $password) or return;
 
-   my $cmd = "winexe -U$user".'%'."$password //$host \"$command\"";
+   # Do not put $command between quotes, let user do it.
+   my $cmd = "winexe -U$user".'%'."$password //$host $command";
+
+   return $self->SUPER::execute($cmd);
+}
+
+sub execute_in_background {
+   my $self = shift;
+   my ($command, $host, $user, $password) = @_;
+
+   $host ||= $self->host;
+   $user ||= $self->user;
+   $password ||= $self->password;
+   $self->brik_help_run_undef_arg('execute_in_background', $command) or return;
+   $self->brik_help_run_undef_arg('execute_in_background', $host) or return;
+   $self->brik_help_run_undef_arg('execute_in_background', $user) or return;
+   $self->brik_help_run_undef_arg('execute_in_background', $password) or return;
+
+   # Do not put $command between quotes, let user do it.
+   my $cmd = "winexe -U$user".'%'."$password //$host $command &";
 
    return $self->SUPER::execute($cmd);
 }
@@ -141,7 +161,7 @@ Metabrik::Remote::Winexe - remote::winexe Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
