@@ -1,5 +1,5 @@
 #
-# $Id: Command.pm,v fc658e60fb76 2016/10/08 15:24:49 gomor $
+# $Id: Command.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # shell::command Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: fc658e60fb76 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(exec execute) ],
       attributes => {
          as_array => [ qw(0|1) ],
@@ -35,7 +35,10 @@ sub brik_properties {
          use_sudo => 0,
          use_pager => 0,
          use_globbing => 0,
-         sudo_args => '-E',  # Keep environment
+         #sudo_args => '-E',  # Keep environment
+         sudo_args => '',     # Do not keep env by default.
+                              # Not needed anymore cause Metabrik is installed
+                              # system-wide.
       },
       commands => {
          system => [ qw(command) ],
@@ -363,7 +366,7 @@ Metabrik::Shell::Command - shell::command Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
