@@ -1,5 +1,5 @@
 #
-# $Id: Csv.pm,v 1cdea52670de 2016/01/02 17:14:57 gomor $
+# $Id: Csv.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # string::csv Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 1cdea52670de $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -20,12 +20,16 @@ sub brik_properties {
          separator => [ qw(character) ],
          header => [ qw($column_header_list) ],
          encoding => [ qw(utf8|ascii) ],
+         escape => [ qw(character) ],
+         quote => [ qw(character) ],
       },
       attributes_default => {
          first_line_is_header => 0,
          header => [ ],
          separator => ';',
          encoding => 'utf8',
+         escape => '"',
+         quote => '"',
       },
       commands => {
          encode => [ qw($data) ],
@@ -76,6 +80,12 @@ sub encode {
          $fields[$order{$key}] = $this->{$key};
       }
 
+      for (@fields) {
+         if (! defined($_)) {
+            $_ = '';
+         }
+      }
+
       my $string = join($self->separator, @fields)."\n";
       print $fd $string;
    }
@@ -100,6 +110,8 @@ sub decode {
       sep_char => $self->separator,
       allow_loose_quotes => 1,
       allow_loose_escapes => 1,
+      escape_char => $self->escape,
+      quote_char => $self->quote,
    }) or return $self->log->error("decode: Text::CSV_XS new failed");
 
    my $fd = IO::Scalar->new(\$data);
@@ -132,7 +144,7 @@ sub decode {
 
    if (! $csv->eof) {
       my $error_str = "".$csv->error_diag();
-      $self->log->error("read: incomplete: error [$error_str]");
+      $self->log->error("decode: incomplete: error [$error_str]");
       return \@rows;
    }
 
@@ -149,7 +161,7 @@ Metabrik::String::Csv - string::csv Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
