@@ -1,5 +1,5 @@
 #
-# $Id: Json.pm,v 8618a4b46183 2016/03/19 16:37:47 gomor $
+# $Id: Json.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # string::json Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 8618a4b46183 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable encode decode) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -32,6 +32,8 @@ sub encode {
    $self->brik_help_run_undef_arg('encode', $data) or return;
    $self->brik_help_run_invalid_arg('encode', $data, 'ARRAY', 'HASH') or return;
 
+   $self->log->debug("encode: data[$data]");
+
    my $encoded = '';
    eval {
       $encoded = JSON::XS::encode_json($data);
@@ -50,6 +52,8 @@ sub decode {
 
    $self->brik_help_run_undef_arg('decode', $data) or return;
 
+   $self->log->debug("decode: data[$data]");
+
    my $decoded = '';
    eval {
       my $j = JSON::XS->new;
@@ -74,7 +78,7 @@ Metabrik::String::Json - string::json Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
