@@ -1,5 +1,5 @@
 #
-# $Id: Syslog.pm,v b5a7d96de9f1 2016/10/11 06:17:23 gomor $
+# $Id: Syslog.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # string::syslog Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: b5a7d96de9f1 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -21,6 +21,7 @@ sub brik_properties {
          process => [ qw(name) ],
          pid => [ qw(id) ],
          do_rfc3164 => [ qw(0|1) ],
+         timestamp => [ qw(timestamp) ],
       },
       attributes_default => {
          process => 'metabrik',
@@ -30,6 +31,7 @@ sub brik_properties {
       commands => {
          encode => [ qw($data hostname|OPTIONAL process|OPTIONAL pid|OPTIONAL) ],
          decode => [ qw($data) ],
+         date => [ qw(timestamp|OPTIONAL) ],
       },
    };
 }
@@ -52,29 +54,33 @@ sub encode {
    $process ||= $self->process;
    $pid ||= $self->pid;
    $self->brik_help_run_undef_arg('encode', $data) or return;
-   $self->brik_help_run_undef_arg('encode', $hostname) or return;
-   $self->brik_help_run_undef_arg('encode', $process) or return;
-   $self->brik_help_run_undef_arg('encode', $pid) or return;
-   $self->brik_help_run_invalid_arg('encode', $data, 'SCALAR') or return;
+   $self->brik_help_set_undef_arg('hostname', $hostname) or return;
+   $self->brik_help_set_undef_arg('process', $process) or return;
+   $self->brik_help_set_undef_arg('pid', $pid) or return;
+   my $ref = $self->brik_help_run_invalid_arg('encode', $data, 'HASH', 'SCALAR')
+      or return;
 
-   my @month = qw{Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec};
+   my $timestamp = $self->timestamp;
 
-   # Courtesy of Net::Syslog
-   my @time = localtime();
-   my $timestamp =
-      $month[$time[4]].
-      ' '.
-      (($time[3] < 10) ? (' '.$time[3]) : $time[3]).
-      ' '.
-      (($time[2] < 10 ) ? ('0'.$time[2]) : $time[2]).
-      ':'.
-      (($time[1] < 10) ? ('0'.$time[1]) : $time[1]).
-      ':'.
-      (($time[0] < 10) ? ('0'.$time[0]) : $time[0]);
+   # Convert to key=value
+   if ($ref eq 'HASH') {
+      my $kv = '';
+      for my $k (sort { $a cmp $b } keys %$data) {
+         if ($k !~ m{\s}) {  # If there is no space char, we don't put between double quotes
+            $kv .= "$k=\"".$data->{$k}."\" ";
+         }
+         else {
+            $kv .= "\"$k\"=\"".$data->{$k}."\" ";
+         }
+      }
+      $kv =~ s{\s*$}{};
+      $data = $kv;
+   }
 
    my $message = '';
    if ($self->do_rfc3164) {
-      $message = "$timestamp $hostname $process\[$pid\]: $data";
+      my $date = $self->date($timestamp);
+      $message = "$date $hostname $process\[$pid\]: $data";
    }
    else {
       $message = "$process\[$pid\]: $data";
@@ -102,6 +108,28 @@ sub decode {
    };
 }
 
+sub date {
+   my $self = shift;
+   my ($timestamp) = @_;
+
+   my @month = qw{Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec};
+
+   # Courtesy of Net::Syslog
+   my @time = defined($timestamp) ? localtime($timestamp) : localtime();
+   my $date =
+      $month[$time[4]].
+      ' '.
+      (($time[3] < 10) ? (' '.$time[3]) : $time[3]).
+      ' '.
+      (($time[2] < 10 ) ? ('0'.$time[2]) : $time[2]).
+      ':'.
+      (($time[1] < 10) ? ('0'.$time[1]) : $time[1]).
+      ':'.
+      (($time[0] < 10) ? ('0'.$time[0]) : $time[0]);
+
+   return $date;
+}
+
 1;
 
 __END__
@@ -112,7 +140,7 @@ Metabrik::String::Syslog - string::syslog Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
