@@ -1,5 +1,5 @@
 #
-# $Id: Xml.pm,v fc658e60fb76 2016/10/08 15:24:49 gomor $
+# $Id: Xml.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # string::xml Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik::System::Package);
 
 sub brik_properties {
    return {
-      revision => '$Revision: fc658e60fb76 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable encode decode) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -40,6 +40,8 @@ sub encode {
 
    my $xs = XML::Simple->new;
 
+   $self->log->debug("encode: data[$data]");
+
    return $xs->XMLout($data);
 }
 
@@ -51,6 +53,8 @@ sub decode {
 
    my $xs = XML::Simple->new;
 
+   $self->log->debug("decode: data[$data]");
+
    return $xs->XMLin($data);
 }
 
@@ -64,7 +68,7 @@ Metabrik::String::Xml - string::xml Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
