@@ -1,5 +1,5 @@
 #
-# $Id: Iocage.pm,v 75fc1a5bf385 2016/09/28 05:57:21 gomor $
+# $Id: Iocage.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # system::freebsd::iocage Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik::Shell::Command Metabrik::System::Package);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 75fc1a5bf385 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -42,8 +42,13 @@ sub brik_properties {
          get_all_properties => [ qw(tag) ],
          get_property => [ qw(tag property) ],
          set_property => [ qw(tag property value) ],
-         backup => [ qw(tag_name|$tag_list) ],
-         restore => [ qw(tag_name) ],
+         backup => [ qw(tag|$tag_list) ],
+         restore => [ qw(tag) ],
+         tag_to_uuid => [ qw(tag) ],
+         get_snapshots => [ qw(tag|OPTIONAL) ],
+         snaplist => [ qw(tag|OPTIONAL) ],  # alias
+         delete_snapshot => [ qw(tag snapshot) ],
+         snapremove => [ qw(tag snapshot) ],  # alias
       },
       require_binaries => {
          iocage => [ ],
@@ -86,6 +91,10 @@ sub list {
          next;
       }
 
+      if (/non iocage jails currently active/) {
+         last;
+      }
+
       my @toks = split(/\s+/, $_);
       my ($ip, $interface) = split(/,/, $toks[5]);
       $ip ||= '';
@@ -307,36 +316,123 @@ sub set_property {
 
 sub backup {
    my $self = shift;
-   my ($tag_name) = @_;
+   my ($tag) = @_;
 
-   $self->brik_help_run_undef_arg('backup', $tag_name) or return;
-   my $ref = $self->brik_help_run_invalid_arg('backup', $tag_name, 'ARRAY', 'SCALAR')
+   $self->brik_help_run_undef_arg('backup', $tag) or return;
+   my $ref = $self->brik_help_run_invalid_arg('backup', $tag, 'ARRAY', 'SCALAR')
       or return;
 
    if ($ref eq 'ARRAY') {
-      for my $tag (@$tag_name) {
+      for my $tag (@$tag) {
          my $cmd = "iocage snapshot \"$tag\"";
          $self->sudo_system($cmd);
       }
       return 1;
    }
 
-   my $cmd = "iocage snapshot \"$tag_name\"";
+   my $cmd = "iocage snapshot \"$tag\"";
 
    return $self->sudo_system($cmd);
 }
 
 sub restore {
    my $self = shift;
-   my ($tag_name) = @_;
+   my ($tag) = @_;
 
-   $self->brik_help_run_undef_arg('restore', $tag_name) or return;
+   $self->brik_help_run_undef_arg('restore', $tag) or return;
 
-   my $cmd = "iocage rollback \"$tag_name\"";
+   my $cmd = "iocage rollback \"$tag\"";
 
    return $self->sudo_system($cmd);
 }
 
+sub tag_to_uuid {
+   my $self = shift;
+   my ($tag) = @_;
+
+   $self->brik_help_run_undef_arg('tag_to_uuid', $tag) or return;
+
+   my $list = $self->list or return;
+   for my $this (@$list) {
+      if ($this->{tag} eq $tag) {
+         return $this->{uuid};
+      }
+   }
+
+   return 'undef';
+}
+
+sub get_snapshots {
+   my $self = shift;
+   my ($tag) = @_;
+
+   # If no tag is given, we will do it for all running iocages
+   my @tags;
+   if (! defined($tag)) {
+      my $list = $self->list or return;
+      for my $this (@$list) {
+         push @tags, $this->{tag};
+      }
+   }
+   else {
+      push @tags, $tag;
+   }
+
+   my @snapshots = ();
+   for my $tag (@tags) {
+      my $cmd = "iocage snaplist $tag";
+      my $lines = $self->sudo_capture($cmd) or return;
+
+      # NAME                                  CREATED                RSIZE   USED
+      # ioc-2016-12-06_16:22:47               Tue Dec  6 16:22 2016  96K    8K
+
+      my $header = 0;
+      for (@$lines) {
+         if (! $header) { # Skip first line, it is a header
+            $header++;
+            next;
+         }
+
+         if (/^(ioc\-\S+)\s+(\S+\s+\S+\s+\S+\s+\S+\s+\S+)\s+(\S+)\s+(\S+)$/) {
+            push @snapshots, {
+               tag => $tag,
+               name => $1,
+               created => $2,
+               rsize => $3,
+               used => $4,
+            };
+         }
+      }
+   }
+
+   return \@snapshots;
+}
+
+# alias
+sub snaplist {
+   my $self = shift;
+
+   return $self->get_snapshots(@_);
+}
+
+sub delete_snapshot {
+   my $self = shift;
+   my ($tag, $snapshot) = @_;
+
+   $self->brik_help_run_undef_arg('delete_snapshot', $tag) or return;
+   $self->brik_help_run_undef_arg('delete_snapshot', $snapshot) or return;
+
+   my $cmd = "iocage snapremove $tag\@$snapshot";
+   return $self->sudo_system($cmd);
+}
+
+# alias
+sub snapremove {
+   my $self = shift;
+
+   return $self->snapremove(@_);
+}
+
 1;
 
 __END__
@@ -347,7 +443,7 @@ Metabrik::System::Freebsd::Iocage - system::freebsd::iocage Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
