@@ -1,5 +1,5 @@
 #
-# $Id: Process.pm,v fc658e60fb76 2016/10/08 15:24:49 gomor $
+# $Id: Process.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # system::process Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik::Shell::Command);
 
 sub brik_properties {
    return {
-      revision => '$Revision: fc658e60fb76 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -19,15 +19,20 @@ sub brik_properties {
          datadir => [ qw(datadir) ],
          force_kill => [ qw(0|1) ],
          close_output_on_start => [ qw(0|1) ],
+         use_pidfile => [ qw(0|1) ],
+         pidfile => [ qw(file) ],
       },
       attributes_default => {
          force_kill => 0,
          close_output_on_start => 1,
+         use_pidfile => 1,
       },
       commands => {
          parse_ps_output => [ qw(data) ],
          list => [ ],
          is_running => [ qw(process) ],
+         is_running_from_string => [ qw(string) ],
+         get_pid_from_string => [ qw(string) ],
          get_process_info => [ qw(process) ],
          kill => [ qw(process|pid) ],
          start => [ qw($sub) ],
@@ -41,6 +46,12 @@ sub brik_properties {
          write_pidfile => [ ],
          delete_pidfile => [ qw(pidfile) ],
          wait_for_pidfile => [ qw(pidfile) ],
+         info_process_is_running => [ ],
+         info_process_is_not_running => [ ],
+         verbose_process_is_running => [ ],
+         verbose_process_is_not_running => [ ],
+         error_process_is_running => [ ],
+         error_process_is_not_running => [ ],
       },
       require_modules => {
          'Daemon::Daemonize' => [ ],
@@ -103,9 +114,35 @@ sub is_running {
    my $list = $self->list or return;
    for my $this (@$list) {
       my $command = $this->{COMMAND};
-      my @toks = split(/\s+/, $command);
-      $toks[0] =~ s/^.*\/(.*?)$/$1/;
-      if ($toks[0] eq $process) {
+      if ($command =~ m{^\[.*\]$}) {   # Example: "[migration/0]"
+         $self->log->debug("is_running: list[$command] process[$process]");
+         if ($command eq $process) {
+            return 1;
+         }
+      }
+      else {
+         my @toks = split(/\s+/, $command);
+         $toks[0] =~ s/^.*\/(.*?)$/$1/;
+         $self->log->debug("is_running: list[$toks[0]] process[$process]");
+         if ($toks[0] eq $process) {
+            return 1;
+         }
+      }
+   }
+
+   return 0;
+}
+
+sub is_running_from_string {
+   my $self = shift;
+   my ($string) = @_;
+
+   $self->brik_help_run_undef_arg('is_running_from_string', $string) or return;
+
+   my $list = $self->list or return;
+   for my $this (@$list) {
+      my $command = $this->{COMMAND};
+      if ($command =~ m{$string}i) {
          return 1;
       }
    }
@@ -113,6 +150,23 @@ sub is_running {
    return 0;
 }
 
+sub get_pid_from_string {
+   my $self = shift;
+   my ($string) = @_;
+
+   $self->brik_help_run_undef_arg('get_pid_from_string', $string) or return;
+
+   my $list = $self->list or return;
+   for my $this (@$list) {
+      my $command = $this->{COMMAND};
+      if ($command =~ m{$string}i) {
+         return $this->{PID};
+      }
+   }
+
+   return 0;
+}
+
 sub get_process_info {
    my $self = shift;
    my ($process) = @_;
@@ -187,11 +241,15 @@ sub start {
       );
    }
 
-   my $pidfile = $self->get_new_pidfile or return;
+   if ($self->use_pidfile) {
+      my $pidfile = $self->get_new_pidfile or return;
 
-   $self->log->verbose("start: new daemon started with pidfile [$pidfile]");
+      $self->log->verbose("start: new daemon started with pidfile [$pidfile]");
 
-   return $pidfile;
+      return $pidfile;
+   }
+
+   return 1;
 }
 
 sub list_daemons {
@@ -283,9 +341,15 @@ sub grep_by_name {
 sub get_new_pidfile {
    my $self = shift;
 
+   # Use provided one.
+   my $pidfile = $self->pidfile;
+   if (defined($pidfile)) {
+      return $pidfile;
+   }
+
    my $id = $self->get_latest_daemon_id;
    defined($id) ? $id++ : ($id = 1);
-   my $pidfile = sprintf("%s/daemonpid.%05d", $self->datadir, $id);
+   $pidfile = sprintf("%s/daemonpid.%05d", $self->datadir, $id);
 
    return $pidfile;
 }
@@ -293,7 +357,12 @@ sub get_new_pidfile {
 sub get_latest_pidfile {
    my $self = shift;
 
-   my $pidfile;
+   # Return user provided one.
+   my $pidfile = $self->pidfile;
+   if (defined($pidfile)) {
+      return $pidfile;
+   }
+
    my $id = $self->get_latest_daemon_id;
    if (defined($id)) {
       $pidfile = sprintf("%s/daemonpid.%05d", $self->datadir, $id);
@@ -369,6 +438,54 @@ sub wait_for_pidfile {
    return $found;
 }
 
+sub info_process_is_running {
+   my $self = shift;
+
+   return $self->log->info("process is running");
+}
+
+sub info_process_is_not_running {
+   my $self = shift;
+
+   return $self->log->info("process is NOT running");
+}
+
+sub verbose_process_is_running {
+   my $self = shift;
+
+   return $self->log->verbose("process is running");
+}
+
+sub verbose_process_is_not_running {
+   my $self = shift;
+
+   return $self->log->verbose("process is NOT running");
+}
+
+sub error_process_is_running {
+   my $self = shift;
+
+   return $self->log->error("process is running");
+}
+
+sub error_process_is_not_running {
+   my $self = shift;
+
+   return $self->log->error("process is NOT running");
+}
+
+# XXX: Broken.
+sub _brik_fini {
+   my $self = shift;
+
+   my $pidfile = $self->get_latest_pidfile;
+   if (-f $pidfile) {
+      $self->delete_pidfile($pidfile);
+   }
+
+   return $self->SUPER::brik_fini;
+}
+
 1;
 
 __END__
@@ -379,7 +496,7 @@ Metabrik::System::Process - system::process Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
