@@ -1,5 +1,5 @@
 #
-# $Id: Service.pm,v cad2514d01b8 2016/02/20 17:35:11 gomor $
+# $Id: Service.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # system::service Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik::Shell::Command);
 
 sub brik_properties {
    return {
-      revision => '$Revision: cad2514d01b8 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable daemon) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -27,7 +27,9 @@ sub brik_properties {
       },
       require_modules => {
          'Metabrik::System::Os' => [ ],
+         'Metabrik::System::Debian::Service' => [ ],
          'Metabrik::System::Ubuntu::Service' => [ ],
+         'Metabrik::System::Centos::Service' => [ ],
       },
       require_binaries => {
          service => [ ],
@@ -44,6 +46,12 @@ sub get_system_service {
    if ($os eq 'ubuntu') {
       $ss = Metabrik::System::Ubuntu::Service->new_from_brik_init($self) or return;
    }
+   elsif ($os eq 'debian') {
+      $ss = Metabrik::System::Debian::Service->new_from_brik_init($self) or return;
+   }
+   elsif ($os eq 'centos') {
+      $ss = Metabrik::System::Centos::Service->new_from_brik_init($self) or return;
+   }
    else {
       $ss = $self;
    }
@@ -196,7 +204,7 @@ Metabrik::System::Service - system::service Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
