@@ -1,5 +1,5 @@
 #
-# $Id: Virtualbox.pm,v fc658e60fb76 2016/10/08 15:24:49 gomor $
+# $Id: Virtualbox.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # system::virtualbox Brik
 #
@@ -11,11 +11,12 @@ use base qw(Metabrik::Shell::Command Metabrik::System::Package);
 
 sub brik_properties {
    return {
-      revision => '$Revision: fc658e60fb76 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
       attributes => {
+         datadir => [ qw(datadir) ],
          capture_mode => [ qw(0|1) ],
          type => [ qw(gui|sdl|headless) ],
       },
@@ -35,16 +36,21 @@ sub brik_properties {
          snapshot_live => [ qw(name snapshot_name description|OPTIONAL) ],
          snapshot_delete => [ qw(name snapshot_name) ],
          snapshot_restore => [ qw(name snapshot_name) ],
-         screenshot => [ qw(name file.png) ],
-         dumpguestcore => [ qw(name file.elf) ],
-         dumpvmcore => [ qw(name file.elf) ],
-         extract_memdump_from_dumpguestcore => [ qw(input output) ],
+         screenshot => [ qw(name output.png|OPTIONAL) ],
+         dumpguestcore => [ qw(name output.elf|OPTIONAL) ],
+         dumpvmcore => [ qw(name output.elf|OPTIONAL) ],
+         extract_memdump_from_dumpguestcore => [ qw(input output.vol|OPTIONAL) ],
          restart => [ qw(name type|OPTIONAL) ],
+         info => [ qw(name) ],
+         is_started => [ qw(name) ],
+         is_stopped => [ qw(name) ],
+         get_current_snapshot_id => [ qw(name) ],
       },
       require_modules => {
          'Metabrik::File::Raw' => [ ],
          'Metabrik::File::Read' => [ ],
          'Metabrik::File::Readelf' => [ ],
+         'Metabrik::System::File' => [ ],
       },
       require_binaries => {
          vboxmanage => [ ],
@@ -86,6 +92,10 @@ sub start {
    $self->brik_help_run_undef_arg('start', $name) or return;
    $self->brik_help_run_undef_arg('start', $type) or return;
 
+   if ($self->is_started($name)) {
+      return $self->log->info("start: VM with name [$name] already started");
+   }
+
    return $self->command("startvm \"$name\" --type $type");
 }
 
@@ -101,6 +111,10 @@ sub stop {
 
    $self->brik_help_run_undef_arg('stop', $name) or return;
 
+   if ($self->is_stopped($name)) {
+      return $self->log->info("start: VM with name [$name] already stopped");
+   }
+
    return $self->command("controlvm \"$name\" poweroff");
 }
 
@@ -119,7 +133,28 @@ sub snapshot_list {
 
    $self->brik_help_run_undef_arg('snapshot_list', $name) or return;
 
-   return $self->command("snapshot \"$name\" list");
+   my $lines = $self->command("snapshot \"$name\" list");
+
+   my @list = ();
+   for my $line (@$lines) {
+      if ($line =~ m{^\s*Name:}) {
+         my ($descr, $id) = $line =~ m{^\s*Name:\s+([^\(]+)\(UUID:\s+([^\)]+)\)};
+         if (defined($descr) && defined($id)) {
+            my $current = 0;
+            if ($line =~ m{\*$}) {
+               $current = 1;
+            }
+            $descr =~ s{\s*$}{};
+            push @list, {
+               name => $descr,
+               uuid => $id,
+               current => $current,
+            };
+         }
+      }
+   }
+
+   return \@list;
 }
 
 sub snapshot_live {
@@ -155,14 +190,14 @@ sub snapshot_restore {
 
 sub screenshot {
    my $self = shift;
-   my ($name, $file) = @_;
+   my ($name, $output) = @_;
 
+   $output ||= $self->datadir."/screenshot.png";
    $self->brik_help_run_undef_arg('screenshot', $name) or return;
-   $self->brik_help_run_undef_arg('screenshot', $file) or return;
 
-   $self->command("controlvm \"$name\" screenshotpng \"$file\"") or return;
+   $self->command("controlvm \"$name\" screenshotpng \"$output\"") or return;
 
-   return $file;
+   return $output;
 }
 
 #
@@ -170,14 +205,19 @@ sub screenshot {
 #
 sub dumpguestcore {
    my $self = shift;
-   my ($name, $file) = @_;
+   my ($name, $output) = @_;
 
+   $output ||= $self->datadir.'/output.elf';
    $self->brik_help_run_undef_arg('dumpguestcore', $name) or return;
-   $self->brik_help_run_undef_arg('dumpguestcore', $file) or return;
 
-   $self->command("debugvm \"$name\" dumpguestcore --filename \"$file\"") or return;
+   if (-f $output) {
+      my $sf = Metabrik::System::File->new_from_brik_init($self) or return;
+      $sf->remove($output) or return;
+   }
 
-   return $file;
+   $self->command("debugvm \"$name\" dumpguestcore --filename \"$output\"") or return;
+
+   return $output;
 }
 
 #
@@ -186,14 +226,19 @@ sub dumpguestcore {
 #
 sub dumpvmcore {
    my $self = shift;
-   my ($name, $file) = @_;
+   my ($name, $output) = @_;
 
+   $output ||= $self->datadir.'/output.elf';
    $self->brik_help_run_undef_arg('dumpvmcore', $name) or return;
-   $self->brik_help_run_undef_arg('dumpvmcore', $file) or return;
 
-   $self->command("debugvm \"$name\" dumpvmcore --filename \"$file\"") or return;
+   if (-f $output) {
+      my $sf = Metabrik::System::File->new_from_brik_init($self) or return;
+      $sf->remove($output) or return;
+   }
 
-   return $file;
+   $self->command("debugvm \"$name\" dumpvmcore --filename \"$output\"") or return;
+
+   return $output;
 }
 
 #
@@ -204,8 +249,8 @@ sub extract_memdump_from_dumpguestcore {
    my $self = shift;
    my ($input, $output) = @_;
 
+   $output ||= $self->datadir.'/output.vol';
    $self->brik_help_run_undef_arg('extract_memdump_from_dumpguestcore', $input) or return;
-   $self->brik_help_run_undef_arg('extract_memdump_from_dumpguestcore', $output) or return;
 
    my $fraw = Metabrik::File::Raw->new_from_brik_init($self) or return;
    my $fread = Metabrik::File::Read->new_from_brik_init($self) or return;
@@ -232,6 +277,11 @@ sub extract_memdump_from_dumpguestcore {
    my $fdin = $fread->open($input) or return;
    $fread->seek($offset) or return;
 
+   if (-f $output) {
+      my $sf = Metabrik::System::File->new_from_brik_init($self) or return;
+      $sf->remove($output) or return;
+   }
+
    my $written = 0;
    my $fdout = $fraw->open($output) or return;
    while (<$fdin>) {
@@ -268,6 +318,78 @@ sub restart {
    return $self->start($name, $type);
 }
 
+sub info {
+   my $self = shift;
+   my ($name) = @_;
+
+   $self->brik_help_run_undef_arg('info', $name) or return;
+
+   my $lines = $self->command("showvminfo \"$name\"") or return;
+
+   my $info = {};
+   if (@$lines > 0) {
+      for my $line (@$lines) {
+         my @t = split(/:/, $line, 2);
+         my $k = $t[0];
+         my $v = $t[1];
+         next unless defined($v);
+         $k =~ s{^\s*}{};
+         $k =~ s{\s*$}{};
+         $v =~ s{^\s*}{};
+         $v =~ s{\s*$}{};
+         if (length($k) && length($v)) {
+            $k =~ s{ }{_}g;
+            $k =~ s{(\(|\))}{}g;
+            $info->{lc($k)} = $v;
+         }
+      }
+   }
+
+   return $info;
+
+}
+
+sub is_started {
+   my $self = shift;
+   my ($name) = @_;
+
+   $self->brik_help_run_undef_arg('is_started', $name) or return;
+
+   my $info = $self->info($name) or return;
+   my $state = $info->{state} || 'undef';
+   if ($state !~ m{powered off}) {
+      return 1;
+   }
+
+   return 0;
+}
+
+sub is_stopped {
+   my $self = shift;
+   my ($name) = @_;
+
+   $self->brik_help_run_undef_arg('is_stopped', $name) or return;
+
+   return ! $self->is_started($name);
+}
+
+sub get_current_snapshot_id {
+   my $self = shift;
+   my ($name) = @_;
+
+   $self->brik_help_run_undef_arg('get_current_snapshot_id', $name) or return;
+
+   my $list = $self->snapshot_list($name) or return;
+
+   for my $this (@$list) {
+      if ($this->{current}) {
+         return $this->{uuid};
+      }
+   }
+
+   return 0;
+}
+
 1;
 
 __END__
@@ -278,7 +400,7 @@ Metabrik::System::Virtualbox - system::virtualbox Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
