@@ -1,5 +1,5 @@
 #
-# $Id: Xrandr.pm,v fc658e60fb76 2016/10/08 15:24:49 gomor $
+# $Id: Xrandr.pm,v f6ad8c136b19 2017/01/01 10:13:54 gomor $
 #
 # xorg::xrandr Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik::Shell::Command Metabrik::System::Package);
 
 sub brik_properties {
    return {
-      revision => '$Revision: fc658e60fb76 $',
+      revision => '$Revision: f6ad8c136b19 $',
       tags => [ qw(unstable) ],
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
@@ -43,6 +43,7 @@ sub brik_properties {
          set_output_resolution => [ qw(resolution) ], # Alias to set_first_output_resolution
          clone_first_to => [ qw(secondary_output) ],
          dual_first_right_of => [ qw(secondary_output) ],
+         clone => [ qw(resolution) ],
       },
       require_binaries => {
          xrandr => [ ],
@@ -444,6 +445,46 @@ sub dual_first_right_of {
    return $self->capture($cmd);
 }
 
+sub clone {
+   my $self = shift;
+   my ($resolution) = @_;
+
+   $self->brik_help_run_undef_arg('clone', $resolution) or return;
+
+   my $list1 = $self->list_first_output_resolutions or return;
+   my $ok = 0;
+   for (@$list1) {
+      if ($_ eq $resolution) {
+         $ok = 1;
+         last;
+      }
+   }
+   if (! $ok) {
+      return $self->log->error("clone: first output does not support ".
+         "resolution [$resolution]");
+   }
+
+   $self->set_first_output_resolution($resolution) or return;
+
+   my $second = $self->get_secondary_output or return;
+   my $list2 = $self->list_secondary_output_resolutions or return;
+   $ok = 0;
+   for (@$list2) {
+      if ($_ eq $resolution) {
+         $ok = 1;
+         last;
+      }
+   }
+   if (! $ok) {
+      return $self->log->error("clone: secondary output does not support ".
+         "resolution [$resolution]");
+   }
+
+   $self->set_secondary_output_resolution($resolution) or return;
+
+   return $self->clone_first_to($second);
+}
+
 1;
 
 __END__
@@ -454,7 +495,7 @@ Metabrik::Xorg::Xrandr - xorg::xrandr Brik
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2014-2016, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2014-2017, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of The BSD 3-Clause License.
 See LICENSE file in the source distribution archive.
